import grpc


class ChannelWrapper:
    def __init__(
        self, app=None, target=None, secure=None, credentials=None, prefix="GRPC"
    ):
        self.app = app
        self.target = target
        self.secure = secure
        self.prefix = prefix
        self.credentials = credentials
        self._channel = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        if not self.target:
            self.target = app.config[f"{self.prefix}_TARGET"]
        if self.secure is None:
            self.secure = app.config.get(f"{self.prefix}_SECURE", False)

        if self.secure:
            if not self.credentials:
                self.credentials = app.config.get("f{self.prefix}_CREDENTIALS")
            if not self.credentials:
                self.credentials = grpc.ssl_channel_credentials()

        # app.teardown_appcontext(self.teardown)

    # def teardown(self, exception):
    #     ctx = _app_ctx_stack.top
    #     if hasattr(ctx, "grpc_channel"):
    #         ctx.grpc_channel.close()

    def _new_channel(self):
        if self.secure:
            return grpc.secure_channel(self.target, credentials=self.credentials)
        return grpc.insecure_channel(self.target)

    @property
    def channel(self):
        if self._channel:
            return self._channel
        self._channel = self._new_channel()
        return self._channel


class StubWrapper:
    def __init__(self, stub_cls, channel_wrapper) -> None:
        self.stub_cls = stub_cls
        self.channel_wrapper = channel_wrapper

    def __getattr__(self, name):
        stub = self.stub_cls(self.channel_wrapper.channel)
        return getattr(stub, name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flask_grpc']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=1.0.0', 'grpcio>=1.0.0', 'protobuf>3.0.0']

setup_kwargs = {
    'name': 'flask-grpc',
    'version': '0.1.1',
    'description': 'A grpc client for Flask.',
    'long_description': 'flask-grpc\n====================\n\nA grpc client for Flask.\n\nUsage\n-------\n\ninit\n\n.. code-block:: python\n\n    channel_wrapper = ChannelWrapper(\n        target="gaea-partner-python-grpc-dev-cuwmnyhgmq-uc.a.run.app:443",\n        secure=True,\n        credentials=grpc.ssl_channel_credentials(),\n    )\n    channel_wrapper.init_app(app)\n\n    health_stub = StubWrapper(health_pb2_grpc.HealthStub, channel_wrapper)\n\nviews\n\n.. code-block:: python\n\n    @app.route("/grpc")\n    def grpc_api():\n        reply = health_stub.Check(health_pb2.HealthCheckRequest())\n        return f"grpc response: {reply}"\n',
    'author': 'codeif',
    'author_email': 'me@codeif.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
